/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.messaging.core.distributed;

import java.io.ObjectOutput;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.Externalizable;

/**
 * A PeerInfo instance contains enough information to allow a peer to establish a logical connection
 * to the peer that sent that information.
 *
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 * @version <tt>$Revision: 464 $</tt>
 *
 * $Id: RemotePeerInfo.java 464 2005-11-22 06:46:49Z ovidiu $
 */
public class RemotePeerInfo implements Externalizable
{
   // Constants -----------------------------------------------------

   // Static --------------------------------------------------------
   
   // Attributes ----------------------------------------------------

   protected PeerIdentity peerIdentity;

   // Constructors --------------------------------------------------

   /**
    * For externalization.
    */
   public RemotePeerInfo()
   {
   }

   public RemotePeerInfo(PeerIdentity peerIdentity)
   {
      this.peerIdentity = peerIdentity;
   }

   // Externalizable implementation ---------------------------------

   public void writeExternal(ObjectOutput out) throws IOException
   {
      peerIdentity.writeExternal(out);
   }

   public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
   {
      peerIdentity = new PeerIdentity();
      peerIdentity.readExternal(in);
   }

   // Public --------------------------------------------------------

   public PeerIdentity getPeerIdentity()
   {
      return peerIdentity;
   }

   // Package protected ---------------------------------------------
   
   // Protected -----------------------------------------------------
   
   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------
}
